@@ -1,6 +1,6 @@
 #!/bin/sh
 #
-# Copyright (C) 2008-2011  Internet Systems Consortium, Inc. ("ISC")
+# Copyright (C) 2008-2015  Internet Systems Consortium, Inc. ("ISC")
 #
 # Permission to use, copy, modify, and/or distribute this software for any
 # purpose with or without fee is hereby granted, provided that the above
@@ -14,17 +14,23 @@
 # OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 # PERFORMANCE OF THIS SOFTWARE.
 
-# $Id: clean.sh,v 1.3.142.5 2011-03-13 23:46:42 tbox Exp $
-
 #
 # Clean up after resolver tests.
 #
 rm -f */named.memstats
-rm -f dig.out dig.*.out.*
+rm -f */named.run
+rm -f */ans.run
+rm -f */*.jdb
+rm -f dig.out dig.out.*
+rm -f dig.*.out.*
 rm -f dig.*.foo.*
 rm -f dig.*.bar.*
+rm -f dig.*.prime.*
+rm -f ns4/tld.db
 rm -f ns6/K*
 rm -f ns6/example.net.db.signed ns6/example.net.db
 rm -f ns6/dsset-example.net. ns6/example.net.db.signed.jnl
-rm -f ns7/server.db ns7/server.db.jnl
-rm -f random.data
+rm -f ns6/to-be-removed.tld.db ns6/to-be-removed.tld.db.jnl
+rm -f ns7/server.db ns7/server.db.jnl ns7/named.conf
+rm -f resolve.out
+rm -f .digrc
