@@ -734,7 +734,7 @@ odbc_get_resultset(const char *zone, const char *record,
 static isc_result_t
 odbc_getField(SQLHSTMT *stmnt, SQLSMALLINT field, char **data) {
 
-	SQLINTEGER size;
+	SQLLEN size;
 
 	REQUIRE(data != NULL && *data == NULL);
 
@@ -763,7 +763,7 @@ odbc_getManyFields(SQLHSTMT *stmnt, SQLSMALLINT startField,
 		   SQLSMALLINT endField, char **retData) {
 
 	isc_result_t result;
-	SQLINTEGER size;
+	SQLLEN size;
 	int totSize = 0;
 	SQLSMALLINT i;
 	int j = 0;
@@ -963,13 +963,16 @@ odbc_process_rs(dns_sdlzlookup_t *lookup, dbinstance_t *dbi)
 /*% determine if the zone is supported by (in) the database */
 
 static isc_result_t
-odbc_findzone(void *driverarg, void *dbdata, const char *name)
+odbc_findzone(void *driverarg, void *dbdata, const char *name,
+	      dns_clientinfomethods_t *methods, dns_clientinfo_t *clientinfo)
 {
 
 	isc_result_t result;
 	dbinstance_t *dbi = NULL;
 
 	UNUSED(driverarg);
+	UNUSED(methods);
+	UNUSED(clientinfo);
 
 	/* run the query and get the result set from the database. */
 	/* if result != ISC_R_SUCCESS cursor and mutex already cleaned up. */
@@ -1007,7 +1010,7 @@ odbc_allowzonexfr(void *driverarg, void *dbdata, const char *name,
 	UNUSED(driverarg);
 
 	/* first check if the zone is supported by the database. */
-	result = odbc_findzone(driverarg, dbdata, name);
+	result = odbc_findzone(driverarg, dbdata, name, NULL, NULL);
 	if (result != ISC_R_SUCCESS)
 		return (ISC_R_NOTFOUND);
 
@@ -1219,12 +1222,15 @@ odbc_authority(const char *zone, void *driverarg, void *dbdata,
 
 static isc_result_t
 odbc_lookup(const char *zone, const char *name, void *driverarg,
-	    void *dbdata, dns_sdlzlookup_t *lookup)
+	    void *dbdata, dns_sdlzlookup_t *lookup,
+	    dns_clientinfomethods_t *methods, dns_clientinfo_t *clientinfo)
 {
 	isc_result_t result;
 	dbinstance_t *dbi = NULL;
 
 	UNUSED(driverarg);
+	UNUSED(methods);
+	UNUSED(clientinfo);
 
 	/* run the query and get the result set from the database. */
 	result = odbc_get_resultset(zone, name, NULL, LOOKUP, dbdata, &dbi);
@@ -1506,7 +1512,14 @@ static dns_sdlzmethods_t dlz_odbc_methods = {
 	odbc_lookup,
 	odbc_authority,
 	odbc_allnodes,
-	odbc_allowzonexfr
+	odbc_allowzonexfr,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
 };
 
 /*%
